#!/usr/bin/perl
# Copyright (c) 2013 SUSE LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

=head1 openqa-clone-job

openqa-clone-job - clone job from local or remote openQA instances

=head1 SYNOPSIS

  openqa-clone-job [OPTIONS] JOBREF [KEY=[VALUE] ...]

  openqa-clone-job https://openqa.opensuse.org/t42

  openqa-clone-job --from https://openqa.opensuse.org/tests/42

  openqa-clone-job --from https://openqa.opensuse.org 42

  openqa-clone-job --from https://openqa.opensuse.org --host openqa.example.com 42

  openqa-clone-job --from localhost --host localhost 42 MAKETESTSNAPSHOTS=1 FOOBAR=


=head1 OPTIONS

=over 4

=item B<--host> HOST

connect to specified host

=item B<--from> HOST

get job from specified host

=item B<--dir> DIR

specify directory where test assets are stored (default /var/lib/openqa/factory)

=item B<--skip-deps>

do not clone parent jobs.

=item B<--skip-chained-deps>

do not clone parent jobs of type chained. This makes the job use the downloaded hdd image instead of running the generator job again.

=item B<--skip-download>

do not try any download. You need to ensure all required assets are provided yourself.

=item B<--within-instance> HOST

a shortcut for C<--skip-download --from HOST --host HOST> to clone a job on a remote instance.

=item B<--show-progress>

display a progress bar of downloading asset

=item B<--parental-inheritance>

provide parental job with variables from command line (they go to child job by default).

=item B<--apikey> <value>

specify the public key needed for API authentication

=item B<--apisecret> <value>

specify the secret key needed for API authentication

=item B<--verbose, -v>

increase verbosity

=item B<--help, -h>

print help

=back

=head1 SYNOPSIS

Clone job from another instance. Downloads all assets associated
with the job. Optionally settings can be modified.

openqa-clone-job https://openqa.opensuse.org/t42

openqa-clone-job --from https://openqa.opensuse.org 42

openqa-clone-job --from https://openqa.opensuse.org --host openqa.example.com 42

openqa-clone-job --from localhost --host localhost 42 MAKETESTSNAPSHOTS=1 FOOBAR=

Call with either a full URL pointing to a test job to clone from or one of
both parameters C<--from> or C<--within-instance>. The job ID can be specified
as part of the URL or as its own parameter.

Any parent jobs (chained or parallel) are also cloned unless C<--skip-deps> or
C<--skip-chained-deps> is specified. If C<--skip-chained-deps> is specified
published assets generated by parent jobs are downloaded to be directly used
instead of generated. Keep in mind that any additional parameters are not
added to the also cloned parent jobs.

=cut

use strict;
use warnings;
use Getopt::Long;
Getopt::Long::Configure("no_ignore_case");
use FindBin;
use lib "$FindBin::RealBin/../lib";
use OpenQA::Script::CloneJob;

my %options;
my $jobid;

sub usage($) {
    my $r = shift;
    eval { require Pod::Usage; Pod::Usage::pod2usage($r); };
    if ($@) {
        die "cannot display help, install perl(Pod::Usage)\n";
    }
}

sub parse_options {
    GetOptions(
        \%options,           "from=s",        "host=s",               "dir=s",
        "apikey:s",          "apisecret:s",   "verbose|v",            "skip-deps",
        "skip-chained-deps", "skip-download", "parental-inheritance", "help|h",
        "show-progress",     "within-instance|w=s",
    ) or usage(1);
    usage(0) if $options{help};

    usage(1) if $options{help} || ($options{'within-instance'} && $options{from});
    if ($options{'within-instance'}) {
        ($options{'within-instance'}, $jobid) = split_jobid($options{'within-instance'});
        $options{'skip-download'} = 1;
        $options{'from'}          = $options{'within-instance'};
        $options{'host'}          = $options{'within-instance'};
    }
    elsif ($options{'from'}) {
        ($options{'from'}, $jobid) = split_jobid($options{'from'});
    }
    $jobid = shift @ARGV unless $jobid;
    die "missing job reference, see --help for usage\n" unless $jobid;
    if (!$options{'from'}) {
        ($options{'from'}, $jobid) = split_jobid($jobid);
    }
    usage(1) unless ($jobid && $options{'from'});
    $options{'dir'}  ||= '/var/lib/openqa/factory';
    $options{'host'} ||= 'localhost';
    $options{'args'} = \@ARGV;
    return $jobid;
}

sub main {
    my ($jobid, $host) = parse_options();
    return unless $jobid;
    clone_job($jobid, \%options);
}

main;

1;
